/* Copyright (c) 2000 Wipro Technologies.  All rights reserved. */

/**********************************************************************
*h*   
*
*   Name:       	dp83815.h
* 
*   Description:  	
*  
*   Author:  
*  
*   History:  
*  
*h*   
**********************************************************************/

#pragma ident   "@(#)$RCSfile: dp83815.h,v $ $Revision: 1.2 $ $Date: 2000/12/13 05:17:14 $ NSC"

#ifndef _DP83815_H
#define _DP83815_H

/*** Includes ***/

/*** Define Constants ***/

/* 
 *	PCI Configuration Register set.
 */
#define CFGID	0x00	/* Configuration Identification Register */
#define CFGCS	0x04	/* Configuration Command and Status Register */
#define CFGRID	0x08	/* Configuration Revision ID Register */
#define CFGLAT	0x0C	/* Configuration Latency Timer Register */
#define CFGIOA	0x10	/* Configuration IO Base Address Register */
#define CFGMA	0x14	/* Configuration Memory Address Register */

/* 18h - 28h	- Reserved (reads return zero)	*/

#define CFGSID	0x2C	/* Configuration Subsystem Identificaton Register */
#define CFGROM	0x30	/* Boot ROM Configuration Register */
#define CAPPTR	0x34	/* Capabilities Pointer Register */
#define CFGINT	0x3C	/* Configuration Interrupt Select Register */
#define PMCAP	0x40	/* Power Management Capabilities Register */
#define PMCSR	0x44	/* Power Management Control and Status Register */

/*
 *	Register Bit definitions
 */

#define	CR	0x00	/* Command Register */
#define	CFG	0x04	/* Configuration and Media Status Register */
#define MEAR	0x08	/* MII/EEPROM Access Register */
#define PTSCR	0x0C	/* PCI Test Control Register */
#define	ISR	0x10	/* Interrupt Status Register */
#define	IMR	0x14	/* Interrupt Mask Register */
#define	IER	0x18	/* Interrupt Enable Register */

#define	TXDP	0x20	/* Transmit Descriptor Pointer Register */

#define	TXCFG	0x24	/* Transmit Configuration Register */

#define	RXDP	0x30	/* Receive Descriptor pointer Register */
#define	RXCFG	0x34	/* Receive Configuration Register */

#define	CCSR	0x3C	/* CLKRUN Control/Status Register */
#define	WCSR	0x40	/* Wake on LAN Control/Status Register */
#define	PCR	0x44	/* Pause Control/Status Register */
#define	RFCR	0x48	/* Receive Filter/Match Control Register */
#define	RFDR	0x4C	/* Receive Filter/Match Data Register */
#define	BRAR	0x50	/* Boot ROM Address Register */
#define	BRDR	0x54	/* Boot ROM Data Register */
#define	SRR	0x58	/* Silicon Revision Register */
#define	MIBC	0x5C	/* Management Information Base Control Register */

#define	TXDP1	0xA0	/* Transmit descriptor for priority queue 1 */
#define	TXDP2	0xA4	/* Transmit descriptor for priority queue 2 */
#define	TXDP3	0xA8	/* Transmit descriptor for priority queue 3 */

#define	RXDP1	0xA0	/* Receive descriptor for priority queue 1 */
#define	RXDP2	0xA4	/* Receive descriptor for priority queue 2 */
#define	RXDP3	0xA8	/* Receive descriptor for priority queue 3 */

#define	BMCR	0x80    /* Basic Mode Control Register */
#define	BMSR	0x84    /* Basic Mode Status Register */
#define	PHYIDR1	0x88    /* PHY Identifier Register #1 */
#define	PHYIDR2	0x8C    /* PHY Identifier Register #2 */
#define ANAR    0x90    /* Auto-Negotiation Advertisement Register */
#define ANLPAR  0x94    /* Auto-Negotiation Link Partner Ability Register */
#define ANER    0x98    /* Auto-Negotiation Expansion Register */
#define ANNPTR  0x9C    /* Auto-Negotiation Next Page TX */

#define PHYSTS  0xC0    /* PHY Status Register */
#define MICR    0xC4    /* MII Interrupt Control Register */
#define MISR    0xC8    /* MII Interrupt Status Register */

#define FCSCR   0xD0    /* False Carrier Sense Counter Register */
#define RECR    0xD4    /* Receive Error Counter Register */
#define PCSR    0xD8    /* 100 Mb/s PCS Configuration and Status Register */

#define PHYCR   0xE4    /* PHY Control Register */
#define TBTSCR  0xE8    /* 10Base-T Status/Control Register */

#define RXErroredPkts	0x60	
#define	RXFCSErrors	0x64
#define	RXMsdPktErrors 	0x68
#define	RXFAErrors	0x6C
#define	RXSymbolErrors	0x70
#define	RXFrameTooLong	0x74
#define	TXSQEErrors	0x88

/*  
 *	CR  - command register bit definitions 
 */

#define TXE          0x00000001      /* transmitter enable */
#define TXD          0x00000002      /* transmitter disable */
#define RXE          0x00000004      /* receiver enable */
#define RXD          0x00000008      /* receiver disable */
#define TXR          0x00000010      /* transmitter reset */
#define RXR          0x00000020      /* receiver reset */

#define	TXDP0_PRISEL 0x00000200
#define	TXDP1_PRISEL 0x00000400
#define	TXDP2_PRISEL 0x00000800
#define	TXDP3_PRISEL 0x00001000

#define	RXDP0_PRISEL 0x00002000
#define	RXDP1_PRISEL 0x00004000
#define	RXDP2_PRISEL 0x00008000
#define	RXDP3_PRISEL 0x00010000

/*		     0x00000040 - 	Unused	*/

#define SWI          0x00000080      /* software interrupt */
#define RST          0x00000100      /* soft reset */

/*	     	bits 17 - 32 unused	*/

/*  
 *	CFG  - Configuration and Media Status Register
 */

#define BEM             0x00000001      /* big endian mode */
#define BROM_DIS        0x00000004      /* disable boot rom interface (unused) */
#define PESEL           0x00000008      /* assert SERR on DPERR detection */
#define EXD             0x00000010      /* disable excessive deferral errs */
#define POW             0x00000020      /* program out-of-window */
#define SB              0x00000040      /* Single Backoff */
#define REQALG          0x00000080      /* PCI bus request algorithm */
#define EUPHCOMP        0x00000100      /* DP83810 Descriptor Compatibility */
#define PHY_DIS         0x00000200      /* disable interal phy */
#define PHY_RST         0x00000400      /* reset internal phy */

/*   bits 11 - 12 Reserved */
/*   Auto Negotiation Select bits to be defined */

#define PAUSE_ADV       0x00010000      /* Pause Advertise */
#define PINT_ACEN       0x00020000      /* Phy Interrupt Auto Clear Enable */
#define ANEG_DN         0x08000000      /* Auto Negotiation Done */
#define POL             0x10000000      /* 10Mb Polarity Indication */
#define FDUP            0x20000000      /* Full Duplex */
#define SPEED100        0x40000000      /* Speed 100Mb */
#define LNKSTS          0x80000000      /* Link Status */

#define	FULL_DUPLEX 0x00
#define	HALF_DUPLEX 0x01

/* 
 * EEPROM Access Register.
 */
#define EEDI            0x00000001      /* EEPROM data in (out data out) */
#define EEDO            0x00000002      /* EEPROM data out (our data in) */
#define EECLK           0x00000004      /* EEPROM serial clock */
#define EESEL           0x00000008      /* EEPROM chip select */

/* 	bits 7-31 - unused	*/

/* 
 * PCSCR - PCI Test Control Register
 */
#define EEBIST_FAIL     0x00000001      /* EE BIST fail indication */
#define EEBIST_EN       0x00000002      /* enable EEPROM BIST */
#define EELOAD_EN       0x00000004      /* enable EEPROM load */
#define RBIST_RXFFAIL   0x00000008      /* RX Filter BIST fail (RO) */
#define RBIST_TXFAIL    0x00000010      /* Hash Filter BIST fail (RO) */
#define RBIST_RXFAIL    0x00000020      /* TX Data FIFO BIST fail (RO) */
#define RBIST_DONE      0x00000040      /* TX FIFO 1 BIST fail (RO) */
#define RBIST_EN        0x00000080      /* RX Data FIFO BIST fail (RO) */
#define RBIST_RST       0x00000400      /* SRAM BIST Reset */

/* 
 *	ISR, IMR , IE , IHR - interrupt bit definitions 
 */
//#define	INTR_MASK	0x00000000	/* Interrupt mask */
#define	INTR_MASK		0xFFFFFFFF	/* Interrupt mask */ // For Checking!
#define RXOK            0x00000001      /* RX OK interrupt */
#define RXDESC          0x00000002      /* RX descriptor interrupt */
#define RXERR           0x00000004      /* RX packet error */
#define RXEARLY         0x00000008      /* RX Early Threshold */
#define RXIDLE          0x00000010      /* RX Idle */
#define RXORN           0x00000020      /* RX Overrun */

#define TXOK            0x00000040      /* TX OK interrupt */
#define TXDESC          0x00000080      /* TX descriptor interrupt */
#define TXERR           0x00000100      /* TX packet error */
#define TXIDLE          0x00000200      /* TX idle */
#define TXURN           0x00000400      /* TX underrun */

#define MIB             0x00000800      /* MIB service */
#define SWI_INTR        0x00001000      /* software interrupt */
#define PME             0x00002000      /* power management event */
#define PHY             0x00004000      /* phy interrupt */
#define HIBERR          0x00008000      /* hi bit error */
#define RXSOVR          0x00010000      /* rx status fifo overrun */
#define RTABT           0x00100000      /* received target abort */
#define RMABT           0x00200000      /* received system abort */
#define SSERR           0x00400000      /* signalled system error */
#define DPERR           0x00800000      /* detected parity error */
#define RXRCMP          0x01000000      /* receive reset complete */
#define TXRCMP          0x02000000      /* transmit reset complete */

/*	bit 31 - 26  Reserved	*/

#define	IE		0x00000001	/* Interrupt Enable */

/* 
 * Transmit Descriptor Pointer , Transmit Descriptor Pointer HighWord Register
 */

#define	TXDP_MASK		0xFFFFFFFC	/* Mask for Transmit Descriptor Pointer	*/

/* 
 * TXCFG - Transmit Configuration Register
 */

/*	Max DMA burst size per Tx/Rx DMA burst	*/

#define	MXDMA4			0x00100000	/* 8 bytes	*/
#define	MXDMA8			0x00200000	/* 16 bytes	*/
#define	MXDMA16			0x00300000	/* 32 bytes	*/
#define	MXDMA32			0x00400000	/* 64 bytes	*/
#define	MXDMA64 		0x00500000	/* 128 bytes	*/
#define	MXDMA128   		0x00600000	/* 256 bytes	*/
#define	MXDMA256		0x00700000	/* 512 bytes	*/
#define	MXDMA512		0x00000000	/* 1024 bytes	*/

#define ECRETRY         0x00800000  /* Excessive Collision Retry Enable */

/* InterFrame Gap Time has to be defined */

#define	ATP		0x10000000  /* Automatic Transmit Padding	*/
#define	HBI		0x40000000  /*	Heart Beat Ignore	*/
#define	CSI		0x80000000  /*	Carrier Sense Ignore	*/

/* 
 * RXCFG - Receive Configuration Register
 */

#define	ALP			0x08000000
#define	RX_FD			0x10000000
#define	ARP			0x40000000
#define AEP			0x80000000

/*
 *	CCSR - CLKRUN Control / Status Register.
 */
#define	CLKRUN_EN		0x00000001
#define	PMEEN			0x00000100
#define	PMESTS			0x00008000

/*
 *	Wake Command / Status Register. 
 */
#define	WKPHY			0x00000001
#define	WKUCP			0x00000002
#define	WKMCP			0x00000004
#define	WKBCP			0x00000008
#define	WKARP			0x00000010
#define	WKPAT0			0x00000020
#define	WKPAT1			0x00000040
#define	WKPAT2			0x00000080
#define	WKPAT3			0x00000100
#define	WKMAG			0x00000200

/*
 *	PCR - Pause Control / Status Register.
 */

/*
 *	Receive Filter / Match Control Register.
 */

#define APAT0			0x00800000
#define APAT1			0x01000000
#define APAT2			0x02000000
#define APAT3			0x04000000
#define RFEN			0x80000000

/*
 *	MIBC - Management Information Base Control Register.
 */
#define	ACLR			0x00000004

/*
 *	RFCR - Receive Flow/Filter Control Register
 */
#define RFAA_shift      28
#define RFEP_shift      16
#define RFEN            0x80000000  /* RF enable */
#define AAB           	0x40000000    /* RF accept all broadcasts */
#define AAM           	0x20000000    /* RF accept all multicasts */
#define AAU           	0x10000000    /* RF accept all unicast */
#define APM             0x08000000  /* accept on perfect match */
#define APAT            0x07800000  /* (&) accept on pattern match-bits 26-23*/
#define AARP            0x00400000      /* accept ARP packets */
#define MHEN            0x00200000      /* mulitcast hash enable */
#define UHEN            0x00100000      /* unicast hash enable */
#define ULM             0x00080000      /* U/L bit mask */

/*
 * BMCR - Basic Mode Control Register
 */
 
#define BMCR_RESET		0x8000

#define AUTONEG_COMPLETE	0x00000010
#define	AUTONEGENABLE		0x00001000
#define RESTARTAUTONEG		0x00000200		

/*** Define Macros ***/

/*** Typedef Definitions ***/

/*** Global (extern) Data Declarations ***/

/*** Global (extern)Functions Declaration ***/

#endif 

