/********************************************************************
*h*
*   Name:	
*
*   		dp83820_ioctl.c
*
*   Description :	
*
*   		This file contains ioctl related functions.
*  
* 
*   History:  
*h*
**********************************************************************/

#include "nsm.h"
#include "hsm.h"

/*********************************************************************
 * Name : dp83820_ioctl
 *
 * Description :
 *
 *	This is the ioctl handler of the driver
 *
 * Arguments :
 *
 *	ifp     This is a pointer to the ifnet structure
 *	command This indicates the ioctl command
 *	data    Address of the data.
 *
 * Return Values
 *
 *	0         On Success
 *	EINVAL    On Error
 *
 *
 *********************************************************************/

INT 
dp83820_ioctl(struct ifnet *ifp,ULONG command,caddr_t data)
{
	NsmContext	*pNsm = ifp->if_softc;
	AdapterContext *pAdapter = &(pNsm->Adapter);
	struct ifreq		*ifr = (struct ifreq *) data;
	INT s, error = 0;
	UCHAR retval;

	s = splimp();

	switch(command) {
	case SIOCSIFADDR:
	case SIOCGIFADDR:
		error = ether_ioctl(ifp, command, data);
		break;
	case SIOCSIFMTU:

/* 		ChangeMtu is called to change the receive buffer size to the
 *		new mtu size
 */
		retval = ChangeMtu(ifp,ifr->ifr_mtu);

#ifdef NSMDEBUG
		printf("Mtu Size is=%d \n",ifr->ifr_mtu);
		if(retval == SUCCESS)
			printf("MTU is equal to receive buffer Size \n");
		else
			printf("MTU is not changed \n");
#endif

		if(retval == SUCCESS){
			ifp->if_mtu = ifr->ifr_mtu;
		}
		else{
			printf("The  MTU change failed  \n");
			error = EINVAL;
		}	
		break;
	case SIOCSIFFLAGS:
		if (ifp->if_flags & IFF_UP) {
			if (ifp->if_flags & IFF_RUNNING &&
			    ifp->if_flags & IFF_PROMISC &&
			    !(pNsm->rxFlags & PROMISCUOUS_ON)) {
				pNsm->rxFlags |= (PROMISCUOUS_ON | ACCEPT_ALL_MULTICAST_ON);
				pNsm->rxFlags &= ~(PROMISCUOUS_OFF);
				HsmRxFilter(pAdapter,pNsm->rxFlags);

			} else if (ifp->if_flags & IFF_RUNNING &&
			    !(ifp->if_flags & IFF_PROMISC) &&
			    pNsm->rxFlags & PROMISCUOUS_ON) {

				pNsm->rxFlags &= ~(PROMISCUOUS_ON);
				pNsm->rxFlags |= (PROMISCUOUS_OFF);
				if (!(ifp->if_flags & IFF_ALLMULTI))
					pNsm->rxFlags &= ~(ACCEPT_ALL_MULTICAST_ON);
				HsmRxFilter(pAdapter,pNsm->rxFlags);

			} else if (!(ifp->if_flags & IFF_RUNNING)){
				dp83820_init(pNsm);
			}
		} else {
			if (ifp->if_flags & IFF_RUNNING)
			{
				dp83820_stop(pNsm);
			}
		}
		pNsm->dp83820_if_flags = ifp->if_flags;
		error = 0;
		break;
	case SIOCADDMULTI:
	case SIOCDELMULTI:
		error = set_multicast(pNsm);
		break;
	case SIOCGIFMEDIA:
	case SIOCSIFMEDIA:
		error = ifmedia_ioctl(ifp, ifr, &pNsm->dp83820_media, command);
		break;
	default:
		error = EINVAL;
		break;
	}

	(void)splx(s);
	return(error);
}

/*********************************************************************
 * Name : dp83820_ifmedia_update
 *
 * Description :
 *
 *	This function sets the speed and duplex mode to the adapter
 *	according to the request.
 *
 * Arguments :
 *
 *	ifp This is a pointer to the ifnet structure
 *
 * Return Values
 *
 *	0       On success
 *	EINVAL  On error.
 *
 *********************************************************************/

INT
dp83820_ifmedia_update(struct ifnet *ifp)
{
	NsmContext *pNsm = ifp->if_softc;
	AdapterContext *pAdapter = &(pNsm->Adapter);
	struct ifmedia *ifm = &(pNsm->dp83820_media);
	INT media;


	media = ifm->ifm_media;

	if (IFM_TYPE(media) !=IFM_ETHER)
	{
		return(EINVAL);
	}

	/*pAdapter->PhysCapabilities &= ~(AUTO_NEG_ON);*/

	switch (media & 0x00F00000)
	{
	case IFM_FDX:
		pAdapter->UserDuplexMode = FULL_DUPLEX;
		break;
	case IFM_HDX:
		pAdapter->UserDuplexMode = HALF_DUPLEX;
		break;
	default:
		pAdapter->PhysCapabilities |= AUTO_NEG_ON;
		pAdapter->UserDuplexMode = FULL_DUPLEX;
		break;
	}

	switch (IFM_SUBTYPE(media))
	{
	case IFM_100_TX:
		pAdapter->UserMediaSpeed = 100;
		break;
	case IFM_10_T:
		pAdapter->UserMediaSpeed = 10;
		break;
	default:
	case IFM_1000_TX:
	case IFM_1000_FX:
		pAdapter->UserMediaSpeed = 1000;
		break;
	}

	HsmSetPhyCapabilities(pAdapter, pAdapter->PhysCapabilities, 1);

	return 0;
}

/*********************************************************************
 * Name : dp83820_ifmedia_status
 *
 * Description :
 *
 *	This function reports the current speed and mode of the adapter.
 *
 * Arguments :
 *
 *	ifp  This is a pointer to the ifnet structure.
 *	ifmr This is a pointer to the if media request structure.
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID
dp83820_ifmedia_status(struct ifnet *ifp,struct ifmediareq *ifmr)
{
	NsmContext *pNsm = ifp->if_softc;
	AdapterContext *pAdapter;

	pAdapter = &(pNsm->Adapter);

	ifmr->ifm_status = IFM_AVALID; /* IFM_ACTIVE will be valid */
	ifmr->ifm_active = IFM_ETHER;

	HsmCheckMediaStatus(pAdapter);

	if (pAdapter->AdapterStatus & LINK_UP)
	{
		ifmr->ifm_status |= (IFM_ACTIVE);
	}
	else
	{
		ifmr->ifm_status &= ~(IFM_ACTIVE);
	}


	if (pAdapter->PhysCapabilities & AUTO_NEG_ON)
		ifmr->ifm_active |= IFM_AUTO;
	else if (pAdapter->DuplexMode & FULL_DUPLEX)
		ifmr->ifm_active |= IFM_FDX;
	else if (pAdapter->DuplexMode & HALF_DUPLEX)
		ifmr->ifm_active |= IFM_HDX;


	switch(pAdapter->MediaSpeed) {
	case 100 :
		ifmr->ifm_active |= IFM_100_TX;
		break;
	case 10 :
		ifmr->ifm_active |= IFM_10_T;
		break;
	case 1000 :
	default :
		if (pAdapter->tbi)
			ifmr->ifm_active |= IFM_1000_FX;
			else
			ifmr->ifm_active |= IFM_1000_TX;
		break;
	}
	return;
}

/*********************************************************************
 * Name : set_multicast 
 *
 * Description :
 *
 * Here we add the list of multicast addresses from the ifnet structure into
 *	the adapter.
 *
 * Arguments :
 *
 *	pNsm	pointer to NsmContext structure.
 *
 * Return Values
 *   0	    - SUCCESS
 *   EINVAL - On Failure
 *
 *********************************************************************/

UCHAR 
set_multicast(NsmContext *pNsm)
{

	struct ifnet		*ifp;
	struct ifmultiaddr *ifma;
	MultiList *pMulti=NULL,*pMultiList=NULL;
	AdapterContext *pAdap = &pNsm->Adapter;
	UCHAR ret_val;

	ifp = &pNsm->arpcom.ac_if;
	for(ifma = ifp->if_multiaddrs.lh_first;ifma !=NULL; ifma=ifma->ifma_link.le_next)
	{
		if(ifma->ifma_addr->sa_family != AF_LINK)
			continue;

		pMulti = (MultiList *) NsmMalloc(sizeof(MultiList));
		if(pMulti == NULL)
		{
			if(pMultiList)
			{
				pMultiList = pMulti->Next;
				NsmFree(pNsm,sizeof(multiList));
				pMulti = pMultiList;
			}
			return FAILURE;
		}
		NsmCopy(LLADDR((struct sockaddr_dl *)ifma->ifma_addr),
		    pMulti->MulticastAddr,6);
		pMulti->Next = pMultiList;
		pMultiList = pMulti;
	}
	ret_val=HsmMulticastAdd(pAdap,pMultiList,TRUE);

	if(ret_val == SUCCESS)
		ret_val=0;

		else
		ret_val = EINVAL;

	pMulti = pMultiList;
	while(pMulti)
	{
		pMultiList = pMulti->Next;
		NsmFree(pMulti,sizeof(multilist));
		pMulti=pMultiList;
	}
	return ret_val;
}

/******************************************************************************
*
*   Name:         ChangeMtu
*
*   Description:
*
*   We Check whether the new mtu is within the required range.
*   If the mtu is within the range then we deallocate all the
*   tranmsit and receive decriptors and receive buffers.
*   The value specified by NewMtu is taken as the size of the 
*   receive buffers and is initialised with that size.
*
*   Arguments:
*
*	ifp          This is a pointer to the ifnet structure
*	NewMtu       The new MTU size which needs to be set in the ifnet
*
*   Return Value: 
*
*	SUCCESS - On Success
*	Failure - On Failure 
*
******************************************************************************/

UCHAR
ChangeMtu(struct ifnet *ifp,UINT NewMtu)
{
	NsmContext *pNsm = ifp->if_softc;
	AdapterContext *pAdapter = &pNsm->Adapter;
	USHORT PktSize = 0;

/*
 *	Set PktSize to NewMtu + ETHER_HEADER_LENGTH,
 *	Since IP layer fragments the packets at (MTU +ETHER_HEADER_LENGTH).
 */
 	PktSize = NewMtu + ETHER_HEADER_LENGTH;

	/* Check for the accepatable range of MTU sizes */
	if((NewMtu > (MAX_JUMBO_FRAME_SIZE - ETHER_HEADER_LENGTH)))
	{
		printf("MTU size should be less than %d \n",(MAX_JUMBO_FRAME_SIZE - ETHER_HEADER_LENGTH));
		return FAILURE;
	}		

	if(PktSize % RXBUF_SIZE_ALIGN)
		PktSize += (RXBUF_SIZE_ALIGN -(PktSize % RXBUF_SIZE_ALIGN));

	if(pAdapter->MaxPktSize != PktSize)
	{
		if(ifp->if_flags & IFF_UP)
		{
			dp83820_stop(pNsm);
/*
 *			 Sleep for some time so as to process any pending
 * 			 interrupts
 */
			NsmSleep(pNsm,1000);
		}

		if(!pNsm->AdapterUnInitialise)
		{	
			pNsm->AdapterUnInitialise = 1;
			HsmUnInitialize(&(pNsm->Adapter));

/*
 * 			Sleep for some time so as to sync
 *			up the frees
 */
			NsmSleep(pNsm,1000);
		}

		if(AdapterInitialise(pNsm, PktSize) == SUCCESS)
		{
			pNsm->AdapterUnInitialise = 0;
			dp83820_init(pNsm);
		}
		else
		{
			printf("\n Memory Allocation failed,Please try a lesser MTU size OR Unload and Reload the driver \n"); 
			return(FAILURE);
		}
	}
	return (SUCCESS);
}
