/*******************************************************************
*h*
*   Name:	
*
*   		dp83820_recv.c
*
*   Description :	
*
*   		This file contains the receive related functions.
* 
*   History:  
*h*
*********************************************************************/

#include <nsm.h>
#include <hsm.h>


VOID 	
NsmMarkRcbFree(HsmRCB *pRcb)
{
	return ;
}

/*********************************************************************
 * Name : NsmRxPackets
 *
 * Description :
 *
 *	This is called by the HSM, to send the received packets in a
 *	particular priority, over to the upper layers
 *
 * Arguments :
 *
 *	pAdapter Pointer to the AdapterContext structure
 *	RcbHead	 Pointer to the first RCB in the list for which 
 *	         receive completion has happened
 *	RcbTail	 Pointer to the RCB which is the next from the last 
 *               RCB in the list for which receive completion has happened
 *	priority Priority queue on which the packet was obtained
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID 	
NsmRxPackets(AdapterContext *pAdapter, HsmRCB *pRCBhead, HsmRCB *pRCBtail, UINT pri)
{
	HsmRCB *pRCB,*pRCBnext;
	struct ether_header	*eh=NULL;
	struct mbuf		*mbf1=NULL,*mbf2=NULL;
	struct ifnet		*ifp=NULL;
	CHAR * data;
	NsmContext 		*pNsm= (NsmContext *)pAdapter->pNsmContext;

	ifp = &pNsm->arpcom.ac_if;
	pRCB= pRCBhead;
	pRCBnext = pRCBtail;

	while ( (pRCB != pRCBnext)) {
		mbf1 = (struct mbuf *) pRCB->pOsCtrlBlk;
		if(pRCB->More){
#ifdef NSMDEBUG
			printf("\n In Receive : More bit set !");
#endif
			while((pRCB->More) && (pRCB != pRCBnext))
			{
				pRCB->Status = RCB_FREE;
				pRCB = (HsmRCB *)pRCB->pNext;
			}
			pRCB->Status = RCB_FREE;
			pRCB = (HsmRCB *)pRCB->pNext;
		}
		else{

			mbf1->m_len=mbf1->m_pkthdr.len =(INT) pRCB->PktSize;

			/* Get the ethernet header for the ether_input function */
			eh = mtod(mbf1, struct ether_header *);

			/* Store the original data pointer */
			data = mbf1->m_data;

			/* Remove the ether header from the mbuf */
			m_adj(mbf1,sizeof(struct ether_header));

			mbf2 = m_devget(mtod(mbf1, char *), mbf1->m_len, 0, ifp, NULL);

			if (mbf2 == NULL) {
				ifp->if_ierrors++;
				goto NEXT;
			}

			/* Increment the packet count */
			ifp->if_ipackets++;
			mbf2->m_pkthdr.csum_flags = 0;

#ifdef RXCHECKSUM
			if (IP_PKT(pRCB))
				mbf2->m_pkthdr.csum_flags |= CSUM_IP_CHECKED;
			if (!(IPCHKSUMERR(pRCB)))
				mbf2->m_pkthdr.csum_flags |= CSUM_IP_VALID;
			if ((TCP_PKT(pRCB) && !(TCPCHKSUMERR(pRCB))) ||
				(UDP_PKT(pRCB) && !(UDPCHKSUMERR(pRCB)))){
				mbf2->m_pkthdr.csum_flags |= CSUM_DATA_VALID | CSUM_PSEUDO_HDR;
				mbf2->m_pkthdr.csum_data = 0xffff;
			}
#endif

#ifdef VLAN_8021P
/*
 *			Vlan Packets are sent through the interface
 * 			vlan_input_tag
 */
			if (IS_VLAN_PACKET(pRCB))
			{
#ifdef NSMDEBUG
				printf("Receive  Vlan tag =%d \n",(ntohs((USHORT)pRCB->ExtSts) & 0xFFFF));
#endif
/*
 *				Convert the vlan tag from network to host 
 *				order for the vlan_input_tag
 */
				vlan_input_tag(eh, mbf2,(ntohs((USHORT)pRCB->ExtSts) & 0xFFFF));
				goto NEXT;
			}
#endif
/*
 *			Ethernet Packets are sent through the interface
 * 			ether_input
 */
			ether_input(ifp, eh, mbf2);

NEXT :
			/* Readjust the original data pointer to previous value */
			mbf1->m_data = data;
			pRCB->Status = RCB_FREE;
			pRCB = (HsmRCB *)pRCB->pNext;
		}
	}
}
