/*******************************************************************
*h*
*   Name:	
*
*   		dp83820_send.c
*
*   Description :	
*
*   		This file contains the send  related functions.
* 
*   History:  
*h*
*********************************************************************/

#include <nsm.h>
#include <hsm.h>

/*********************************************************************
 * Name : FillPktStruct 
 *
 * Description :
 *
 *	This function  encapsulate the mbs  data into PktInfo
 *
 * Arguments :
 *
 *	mb      is a pointer to the mbuf
 *	pktInfo is a pointer to the pktInfo.
 *
 * Return Values
 *
 *	0        On Success
 *	ENOBUFS  On greater than MAX_FRAGS
 *
 *********************************************************************/

INT 
FillPktStruct(
struct mbuf	*m_head,
PktInfo *pktInfo
)
{
	struct mbuf	*mb;
	INT fragCnt=0,dataLength=0;
 	UINT CheckSum=0;
	struct ifvlan	*ifv = NULL;

#ifdef VLAN_8021P
	if ((m_head->m_flags & (M_PROTO1|M_PKTHDR)) == (M_PROTO1|M_PKTHDR) &&
			m_head->m_pkthdr.rcvif != NULL &&
			m_head->m_pkthdr.rcvif->if_type == IFT_8021_VLAN) {
		ifv = m_head->m_pkthdr.rcvif->if_softc;
#ifdef NSMDEBUG
		printf("Send Vlan tag =%d \n",ifv->ifv_tag);
#endif
	}
#endif
	mb = m_head;

	for (mb = m_head; mb != NULL; mb = mb->m_next) {
		if (mb->m_len != 0) {
			fragCnt++;
		}
	}

	if (fragCnt >= MAX_FRAGS)
	{
		return(ENOBUFS);
	}

	fragCnt = 0;

	for (mb = m_head; mb != NULL; mb = mb->m_next) {
		if (mb->m_len != 0) {
			pktInfo->FragArray[fragCnt].pFrag = (UCHAR *)vtophys(mtod(mb,vm_offset_t));
			pktInfo->FragArray[fragCnt].FragLen= mb->m_len;
			fragCnt++;
			dataLength += mb->m_len;
		}
	}

#ifdef TXCHECKSUM
	if (m_head->m_pkthdr.csum_flags) {
		if (m_head->m_pkthdr.csum_flags & CSUM_IP)
			CheckSum |= IPPKT;
		if (m_head->m_pkthdr.csum_flags & CSUM_TCP)
			CheckSum |= TCPPKT;
		if (m_head->m_pkthdr.csum_flags & CSUM_UDP)
			CheckSum |= UDPPKT;
	}
#endif

#ifdef VLAN_8021P
	pktInfo->VLANId = 0;
	pktInfo->IEEE8021Priority = 0;
	if (ifv != NULL){
/*
 *		Convert the vlan tag from host to network order
 *		the vlan tag is a 16 bit value
 */
		pktInfo->VLANId = htons((USHORT)ifv->ifv_tag) & 0xFFFF;
	}
#endif
	pktInfo->NumFrags = fragCnt;
	pktInfo->TotalDataLength = dataLength;
	pktInfo->pOSCtrlBlk = m_head;
	pktInfo->ChkSum = CheckSum;
	return 0;

}

/*********************************************************************
 * Name : dp83820_send
 *
 * Description :
 *
 *	This entry point sends the  packets down to the HSM, to be 
 *	sent on wire.
 *
 * Arguments :
 *
 *	ifp	This is the pointer to the ifnet which contains the packet.
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID 
dp83820_send(
struct ifnet *ifp
)
{
	NsmContext	*pNsm;
	AdapterContext	*pAdapter;
	struct  mbuf *m_head=NULL;
	PktStruct	HsmpktStruct;
	INT pktCount=0,pktSent,i;

	pNsm =(NsmContext *) ifp->if_softc;

	pAdapter = &pNsm->Adapter;

	while((ifp->if_snd.ifq_head != NULL) && (pktCount < MAX_PKTS))
	{
		IF_DEQUEUE(&ifp->if_snd, m_head);
		if (m_head == NULL)
			continue;
		if (FillPktStruct(m_head,&HsmpktStruct.PktArray[pktCount]) == 
		ENOBUFS) 
		{
#ifdef NSMDEBUG
		printf("\n Got more than MAX FRAGS");
#endif
			m_freem(m_head);
			IF_DROP(&ifp->if_snd);
			continue;
		}
		/* if there's a BPK listener, bounce a copy of this frame 
				 * to him */
		if (ifp->if_bpf)
			bpf_mtap(ifp, m_head);
		pktCount++;
	}
	HsmpktStruct.PktCount = pktCount;
	if (pktCount > 0)
	{
		pktSent = HsmSendPkts(pAdapter,&HsmpktStruct,0);
		if (pktSent < pktCount)
		{
#ifdef NSMDEBUG
			printf("\n re-queuing : pkt sent = %d, pkt co = %d",pktSent,pktCount);
#endif
			for (i=pktSent;i<pktCount;i++)
				IF_PREPEND(&ifp->if_snd,
				    (struct mbuf *)HsmpktStruct.PktArray[i].pOSCtrlBlk);
			ifp->if_flags |= IFF_OACTIVE;
		}
		ifp->if_timer = 5;
	}
	return;
}

/*********************************************************************
 * Name : NsmTxCompleted
 *
 * Description :
 *
 *	This is called by the HSM, to send the received packets in a
 *	particular priority, over to the upper layers
 *
 * Arguments :
 *
 *	pAdapter Pointer to the AdapterContext structure
 *	RcbHead  Pointer to the first RCB in the list for 
 *	         which receive completion has happened
 *	RcbTail  Pointer to the RCB which is the next from the 
 *	         last RCB in the list for which receive completion
 *	         has happened
 *	priority Priority queue on which the packet was obtained
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID
NsmTxCompleted(
AdapterContext *pAdapter, 
HsmTCB	*pTCBHead,
HsmTCB *pTCBNext)
{
	HsmTCB	*pTCBcurr;
	NsmContext *pNsm;
	struct ifnet *ifp;

	pNsm = (NsmContext *) pAdapter->pNsmContext;
	ifp = &(pNsm->arpcom.ac_if);
	
	pTCBcurr = pTCBHead;
	
	ifp->if_timer = 0;

	do
	{
		m_freem(pTCBcurr->pOsCtrlBlk);
		pTCBcurr->pOsCtrlBlk = NULL;
		pTCBcurr = pTCBcurr->pNext;
	} while (pTCBcurr != pTCBNext);

	ifp->if_flags &= ~IFF_OACTIVE;

	if(ifp->if_snd.ifq_head != NULL)
	{
		dp83820_send(ifp);
	}
}
