/**********************************************************************
*h*   
*
*   Name:       	nsm.h
* 
*   Description:  	
*		  
*  
*   Author:  
*  
*   History:  
*  
*h*   
**********************************************************************/
#ifndef _NSM_H
#define _NSM_H

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/mbuf.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#include <sys/socket.h>

#include <net/if.h>
#include <net/if_dl.h>
#include <net/if_media.h>
#include <net/bpf.h>

#include <sys/bus.h>
#include <sys/sockio.h>
#include <machine/bus.h>
#include <sys/rman.h>
#include <machine/resource.h>


#include <dev/mii/mii.h>
#include <dev/mii/miivar.h>

#include <net/ethernet.h>
#include <net/if_arp.h>

#include <net/if_types.h>
#include <net/if_vlan_var.h>

#include <vm/vm.h>		/* for vtophys */
#include <vm/pmap.h>		/* for vtophys */
#include <machine/clock.h>	/* for DELAY */

#include <pci/pcivar.h>
#include <pci/pcireg.h>		/* for PCIM_CMD_xxx */
#include <posix4/sched.h>
#include <pthread.h>


#define   MAX_PKTS 10 
#define   MAX_FRAGS 20
#define   LED_TIMER_DELAY 1 /* Time in Seconds */
#define   CHECKSUM_FEATURES (CSUM_IP  | CSUM_TCP | CSUM_UDP)

#define   IEEE_NON_COMPLIANCE 0 	/* IEEE Compliance definitions*/
#define   IEEE_COMPLIANCE 1

#define   OS_PRIO 1			/* NO of Priority Supported in the OS */

/* Vlan Options */
#define   VLAN_INSERTION_OFF 0		/* vlan packets are not accepted */
#define   VLAN_INSERTION_NIC 1		/* Enable the vlan tagging in the NIC */
#define   VLAN_INSERTION_OS 2		/* Enable the OS encapsulation of vlan */
#define   MAX_JUMBO_FRAME_SIZE 16384	/* MAX JUMBO FRAME */


#include "nsmtypes.h"
#include "hsm.h"

/*
 *  Configuration register set bit mask definitions.
 */

#define VENDOR_ID_MASK  0x0000FFFF
#define DEVICE_ID_MASK  0xFFFF0000
#define VENDOR_ID       0x0000100B
#define DEVICE_ID       0x00220000

#define IOSEN           0x00000001
#define MSEN            0x00000002
#define BMEN            0x00000004

#define IOBASE_MASK     0xFFFFFF00
#define IOSIZE_MASK     0x000000FC
#define IOIND           0x00000001

#define MEMSIZE_MASK    0x00000FF0
#define MEMIND          0x00000001
/* Failure condition defintions */
#define  DETACH 0x1 
#define  MEMORY_IOPORT_FAILURE  0x2
#define  INTERRUPT_RESOURCE_FAILURE 0x3
#define  SETUP_INTR_FAILURE 0x4 
#define  HSMINIT_FAILURE 0x5 
#define  HSMOPEN_FAILURE 0x6 
#define  HSMRESET_FAILURE 0x7 


static const int dp83820_media_standard[] = {
			IFM_ETHER|IFM_10_T|IFM_HDX,
			IFM_ETHER|IFM_10_T|IFM_FDX,
			IFM_ETHER|IFM_100_TX|IFM_HDX,
			IFM_ETHER|IFM_100_TX|IFM_FDX,
			IFM_ETHER|IFM_10_T|IFM_AUTO,
			IFM_ETHER|IFM_100_TX|IFM_AUTO,
			IFM_ETHER|IFM_1000_TX|IFM_AUTO,
			IFM_ETHER|IFM_1000_FX|IFM_AUTO,
			IFM_ETHER|IFM_AUTO,
};

#define	DP83820_MEDIA_STANDARD_DEFMEDIA	(IFM_ETHER|IFM_AUTO)

typedef struct NsmContext_t {
	struct arpcom  arpcom; /* Per Interface Network Data */
	AdapterContext		Adapter;	/* Pointer to Adapter Structure */
	UCHAR *instance;
	INT dp83820_if_flags;

	struct resource	*irq;   /*Resource for interrrupt */
	UINT unit;     /* Unique no is assigned to each interface */
	VOID *Irqhandle; 	/*Interrupt handler cookie */

	struct resource	*regmap;/*Resource for Register */
	bus_space_tag_t	bus_tag;/*Bus space tag*/
	bus_space_handle_t bus_handle;/*Bus space handle*/
	INT mapping;
	struct ifmedia dp83820_media;
	UINT rxFlags;
	UINT CacheLineSize;
	struct callout_handle tbi_led_handle;	/*Handle for cancelling our tbi led activity */
	CHAR tbi_activity;	/* Flag to indicate tbi activity */
	CHAR AdapterUnInitialise;
}NsmContext;

/* Functions */
extern   void dp83820_intr(void  *);
int dp83820_attach (device_t);
int dp83820_detach (device_t);
int dp83820_suspend (device_t);
int dp83820_resume (device_t);
void dp83820_shutdown (device_t );
int dp83820_probe (device_t );
extern VOID dp83820_send(struct ifnet *);
extern int dp83820_ioctl (struct ifnet *, ULONG, caddr_t);
void dp83820_init (void *);
void dp83820_stop (NsmContext *);
void dp83820_watchdog (struct ifnet *);
void dp83820_mbuffree (caddr_t,UINT);
void dp83820_mbufref (caddr_t,UINT);
extern int dp83820_ifmedia_update (struct ifnet *);
extern void dp83820_ifmedia_status (struct ifnet *, struct ifmediareq *);
/* #define ether_ioctl fxp_ether_ioctl*/


/*** end of functions ***/

/* Macros to Read and write Registers */
#define	NsmRegRead8(pNsm, reg,val) \
	*val= bus_space_read_1(( \
	      (bus_space_tag_t)((NsmContext *)pNsm)->bus_tag), \
	      (bus_space_handle_t)((NsmContext *) pNsm)->bus_handle, \
	      (INT)reg)

#define	NsmRegRead16(pNsm, reg,val) \
	*val= bus_space_read_2(( \
	      (bus_space_tag_t)((NsmContext *)pNsm)->bus_tag), \
	      (bus_space_handle_t)((NsmContext *) pNsm)->bus_handle, \
	      (INT)reg)

#define	NsmRegRead32(pNsm, reg,val) \
	*val= bus_space_read_4(( \
	      (bus_space_tag_t)((NsmContext *)pNsm)->bus_tag), \
	      ((bus_space_handle_t)((NsmContext *) pNsm)->bus_handle), \
	      (INT)reg)

#define	NsmRegWrite8(pNsm, reg, val) \
	bus_space_write_1((bus_space_tag_t)((NsmContext *)pNsm)->bus_tag, \
	                  (bus_space_handle_t)((NsmContext *)pNsm)->bus_handle,(UINT8 *)reg, \
	                  (UINT8) (val))

#define	NsmRegWrite16(pNsm, reg, val) \
	bus_space_write_2((bus_space_tag_t)((NsmContext *)pNsm)->bus_tag, \
	                  (bus_space_handle_t)((NsmContext *)pNsm)->bus_handle,(UINT16 *) reg, \
	                  (UINT16) (val))

#define	NsmRegWrite32(pNsm, reg, val) \
	bus_space_write_4((bus_space_tag_t)((NsmContext *)pNsm)->bus_tag, \
	                  (bus_space_handle_t)((NsmContext *)pNsm)->bus_handle,(INT)reg, \
	                  val)



/* #define NsmMalloc(size) malloc(sizeof(size), M_DEVBUF, M_NOWAIT)*/
/*#define NsmMalloc(size) malloc(sizeof(size), M_DEVBUF, M_WAITOK)*/
#define NsmFree(buf,size) free((PVOID)buf, M_DEVBUF)
#define NsmReleaseDPCLock(pAdapter,lock)
#define NsmAcquireDPCLock(pAdapter,lock)
#define NsmIndicateLinkStatus(pAdapter)
#define NsmResetComplete(pAdapter)
#define NsmCopy(Src, Dest, Len) bcopy((PVOID) Src, (PVOID) Dest, Len);
#define NsmZeroMemory(Buf, Size)  bzero((PVOID)Buf, Size);
#define NSMGETNUMFRAGS(NumFrags, Ptr) ((NumFrags) = (Ptr)->NumFrags)
#define NSMMAPPKTS(pAdapter, pPkts, prio, NumDesc, PktIndex)

VOID NsmSleep(NsmContext *pNsmContext, int MilliSecs);
VOID NsmFreeContiguous  (NsmContext *pNsm, AddrStruct *pAddr, UINT cached);
VOID NsmMallocContiguous  (NsmContext *pNsm, AddrStruct *pAddr, UINT align, UINT cached);
VOID NsmTxCompleted  (AdapterContext *pAdapter, HsmTCB *pTCBHead, 
HsmTCB *pTCBNext);
UCHAR NsmAllocatePacket (NsmContext *pNsmContext, HsmRCB *pRCB,AddrStruct *pAddr );
VOID NsmDeAllocatePacket  (NsmContext *pNsmContext, AddrStruct *pAddr, 
HsmRCB *pRCB);
VOID NsmRxPackets (AdapterContext *pAdapterContext, HsmRCB *pRCBhead, 
HsmRCB *pRCBnext, UINT pri);
CHAR NsmCompare (PVOID str1, PVOID str2, UINT len);
INT  FillPktStruct(struct mbuf *m_head, PktInfo *pktInfo);
VOID NsmMarkRcbFree (HsmRCB *pRcb) ;
PVOID NsmMalloc (UINT size);
VOID  NsmFreeResources(device_t dev,NsmContext *pNsm,int cmd);
VOID FreeResources(device_t dev);
UCHAR set_multicast(NsmContext *pNsm);
VOID  NsmStartLedWithTimer(NsmContext *pNsm);
VOID  LedTimerFunc(PVOID arg);
VOID NsmGetPkt(NsmContext *pNsmContext, PVOID pOsCtrlBlk,
				UCHAR *pPkt, UCHAR *pProtocolChecksumoffset[],
				USHORT *pPktLength, UINT32 Protocol,
				UINT32 *pNumberOfDescriptors);
UCHAR AdapterInitialise(NsmContext *pNsm,UINT PktSize);
UCHAR ChangeMtu(struct ifnet *ifp,UINT NewMtu);

#define NsmCreateLock() 0xffff
#define NsmDestroyLock(pNsm, lock)
#define NsmAcquireLock(pNsm, lock)
#define NsmReleaseLock(pNsm, lock)
#define NsmEnableInterrupts(pAdapter) HsmEnableInterrupts(pAdapter)
#define NsmDisableInterrupts(pAdapter) HsmDisableInterrupts(pAdapter)
#define NsmUpdateSpeed(pNsm)

#define NsmDbgMsg(str) printf(str)
#define NsmDbgMsg1(str,val) printf(str,val)
#endif
