/**********************************************************************
*h*   
*
*   Name:       	nsm.h
* 
*   Description:  	
*		  
*  
*   Author:  
*  
*   History:  
*  
*h*   
**********************************************************************/
#include <stddef.h>
#include <sys/int_types.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/mutex.h>
#include <sys/stat.h>
#include <sys/systm.h>
#include <sys/sysmacros.h>
#include <sys/stropts.h>
#include <sys/devops.h>
#include <sys/errno.h>
#include <sys/ddi.h>
#include <sys/dlpi.h>
#include <sys/ksynch.h>
#include <sys/modctl.h>
#include <sys/buf.h>
#include <sys/open.h>
#include <sys/poll.h>
#include <sys/kmem.h>
#include <sys/ethernet.h>
#include <sys/conf.h>
#include <sys/cred.h>
#include <sys/cmn_err.h>
#include <sys/stream.h>
#include <sys/sunddi.h>
#include <sys/gld.h>
#include <sys/gldpriv.h>
#include <ctype.h>
#include <sys/debug.h>
#include <sys/ddidmareq.h>
#include <sys/pci.h>

#define   MAX_PKTS 5
#define   MAX_FRAGS 20

#include <nsmtypes.h>
#include <hsm.h>

/*** External Global Variables ***/

extern uint_t * TxQueueSz;
extern uint_t * RxQueueSz;
extern uint_t TxDrth[];
extern uint_t RxDrth[];
extern uint_t TxFlth[];
extern uint_t uMediaSpeed[];
extern uint_t uDuplexMode[];
extern uint_t PauseCounterVal[];
extern uint_t NcBit[];
extern uint_t RxFFLO[];
extern uint_t RxFFHI[];
extern uint_t RxSTLO[];
extern uint_t RxSTHI[];
extern uint_t JumboFrameSize[];
extern uint_t IntrHoldoff[];

/*** External Global Func ***/


#define DP83820_MODULE_ID 	0		/* Module id 	*/
#define DP83820_MINPKT		0			/* Minimum Packet size */
#define DP83820_MAXPKT		1518		/* Maximum packet size */
#define DP83820_HIWAT		(16 * 1024)	/* Hi water Mark */
#define DP83820_LOWAT		1024		/* Low water Mark */

#define	MAXFRAGS		20

/*
 *  Configuration register set bit mask definitions.
 */

#define VENDOR_ID_MASK  0x0000FFFF
#define DEVICE_ID_MASK  0xFFFF0000
#define VENDOR_ID       0x0000100B
#define DEVICE_ID       0x00220000

#define IOSEN           0x00000001
#define MSEN            0x00000002
#define BMEN            0x00000004

#define IOBASE_MASK     0xFFFFFF00
#define IOSIZE_MASK     0x000000FC
#define IOIND           0x00000001

#define MEMSIZE_MASK    0x00000FF0
#define MEMIND          0x00000001

#define ILINE_MASK      0x000000FF


#define	TRUE	0x1
#define	FALSE	0x0

#define IRQ_MASK 		0x00FF

#ifdef OLD
#define RX_QUEUE_SZ		100
#define TX_QUEUE_SZ		100
#endif

/*
 * 	NSM context Structure
 */
typedef struct NsmCookie_t	{ 
	ddi_dma_handle_t	dma_handle;
	ddi_acc_handle_t	dma_acc_handle;
	mblk_t				*pMblk;
} NsmCookie;



typedef struct NsmContext_t {

	gld_mac_info_t		*macinfo;	/* Pointer to gld_mac_info_t structure	*/
	AdapterContext		*pAdapter;	/* Pointer to Adapter Structure */
	dev_info_t			*dip;	/* Pointer to Adapter Structure */
	INT					instance;	/* Device instance	*/
	ddi_acc_handle_t	pci_handle;	/* Handle to PCI config space	*/
	caddr_t				regmap_addr;	
	PVOID				regmap_handle;	
	PVOID				NsmGenLock_cookie;	
	kmutex_t			NsmGenLock;	
	ULONG				IOBase;
	ULONG				MemBase;
	frtn_t				*free_rtn;
	NsmCookie			*DmaHandle;
	int					RcbCnt; /* Track FreeRtn calls for esballoc()ed buffers during detach()*/ 
}NsmContext;

typedef struct ioctl_t {
	ULONG	cmd;
	USHORT	buf[256];
	ULONG	arg1;
	ULONG	arg2;
} nsm_ioctl_t;
	


#define	MAX_ADAPTERS	1
#define	OS_PRIQUE	1
#define	MAX_ETH_PKT_SIZE	1514
#define	RX_BUF_SIZE	2048

#ifdef PAUSE_FRAMES
#define	MAX_PACKET_SIZE	8096
#else
#define	MAX_PACKET_SIZE	MAX_ETH_PKT_SIZE
#endif

#define	ETH_HDR_SIZE	14

#define	REGMAP_FAILURE		1
#define	IBLOCK_COOKIE		2
#define	DETACH			5
#define	INTR_PROP		6
#define	PCI_FAILURE		7
#define	ADAPTER_FOUND		8
#define	ADAPTER_NOT_FOUND	9
#define	HSMINIT_FAILURE		3
#define	HSMOPEN_FAILURE		10
#define	HSMRESET_FAILURE	11
#define	ADD_INTR		12

#define	TOOMANY_FRAGS	0x2
#define	BADPACKET	0x4

#define BUSMASTER_ENABLE_MASK 0x0002
#define MEMSPACE_ENABLE_MASK 0x0001
#define IOSPACE_ENABLE_MASK 0x0000

#ifdef NOTYET
#define PAGEMASK	0xFFFFF000
#define	PAGEOFFSET	0x00000FFF
#define	PAGESIZE	0x00001000
#endif

/* #define GET_VPAGE(var, ptr) (var = ((ulong)(ptr) & PAGEMASK) ) */
#define GET_VPAGE(ptr) ((caddr_t)((ulong)(ptr) & PAGEMASK) )
#define GET_PPAGE(ptr) ((paddr_t)(( hat_getkpfnum((caddr_t)ptr) * 4096 ) + ((uint)ptr & 0x0fff)) )

#define IE_ENABLE_MASK	0x00000001


#define	ADD_MCAST	0x01
#define	DELETE_MCAST	0x02
#define	CHANGE_DATARATE	0x04

//#define NsmMalloc(size, flags) ((PVOID) kmem_zalloc(size, flags))
#define NsmMalloc(size) ((PVOID) kmem_zalloc(size, KM_NOSLEEP))

#define NsmFree(buf, size) (kmem_free((buf), size))

/*
#define	NsmRegRead8(pNsmContext, RegAddr, Val)	\
	*Val = (UINT8) ddi_get8((ddi_acc_handle_t) ((NsmCookie *)((NsmContext *)pNsmContext)->DmaHandle)->dma_acc_handle, (UINT8 *) RegAddr);

#define	NsmRegRead16(pNsmContext, RegAddr, Val)	\
	*Val = (UINT16) ddi_get16((ddi_acc_handle_t) ((NsmCookie *)((NsmContext *)pNsmContext)->DmaHandle)->dma_acc_handle, (UINT16 *) RegAddr);

#define	NsmRegRead32(pNsmContext, RegAddr, Val)	\
	*Val = (UINT32) ddi_get32((ddi_acc_handle_t) ((NsmCookie *)((NsmContext *)pNsmContext)->DmaHandle)->dma_acc_handle, (UINT32 *) RegAddr);

#define	NsmRegWrite8(pNsmContext, RegAddr, Val)	\
	ddi_put8((ddi_acc_handle_t) ((NsmCookie *)((NsmContext *)pNsmContext)->DmaHandle)->dma_acc_handle, (UINT8 *) RegAddr, (UINT8) Val);

#define	NsmRegWrite16(pNsmContext, RegAddr, Val)	\
	ddi_put16((ddi_acc_handle_t) ((NsmCookie *)((NsmContext *)pNsmContext)->DmaHandle)->dma_acc_handle, (UINT16 *) RegAddr, (UINT16) Val);

#define	NsmRegWrite32(pNsmContext, RegAddr, Val)	\
	ddi_put32((ddi_acc_handle_t) ((NsmCookie *)((NsmContext *)pNsmContext)->DmaHandle)->dma_acc_handle, (UINT32 *) RegAddr, (UINT32) Val);
*/

#define NsmRegRead8(pNsmContext, RegAddr, Val)  *Val = (UCHAR) inb((INT)RegAddr)
#define NsmRegRead16(pNsmContext, RegAddr, Val) *Val = (USHORT) inw((INT)RegAddr)
#define NsmRegRead32(pNsmContext, RegAddr, Val) *Val = (UINT) inl((INT)RegAddr)

#define NsmRegWrite8(pNsmContext, RegAddr, Val) outb((INT)RegAddr, (UCHAR)Val)
#define NsmRegWrite16(pNsmContext, RegAddr, Val) outw((INT)RegAddr, (USHORT)Val)
#define NsmRegWrite32(pNsmContext, RegAddr, Val) outl((INT)RegAddr, (UINT)Val)

#define NsmDestroyLock(pNsmContext, lock)	\
	{	\
        	mutex_destroy((kmutex_t *)lock);	\
        	kmem_free(lock, sizeof(kmutex_t));	\
	}

#define NsmAcquireLock(pNsmContext, lock)	mutex_enter((kmutex_t *)lock)

#define NsmAcquireDPCLock	NsmAcquireLock

#define NsmReleaseLock(pNsmContext, lock)	mutex_exit((kmutex_t *)lock)

#define NsmReleaseDPCLock	NsmReleaseLock

#define	NsmCopy(Src, Dest, Len)	bcopy((PVOID) Src, (PVOID) Dest, Len);

//#define	NsmSleep(pNsmConetxt, Secs)	delay(drv_usectohz(Secs * 1000 * 1000));
#define	NsmSleep(pNsmConetxt, MilliSecs)	delay(drv_usectohz(MilliSecs * 1000));

#define	NsmZeroMemory(Buf, Size) 	bzero((PVOID)Buf, Size);

#define	NsmDbgMsg(Str)	cmn_err(CE_NOTE, Str);

#define	NsmDbgMsg1(Str, Val)	cmn_err(CE_NOTE, Str, Val);

/*#define NsmMarkRcbFree 	*/

#define	NsmResetComplete

//#define	CMP_MAC(Str1, Str2)	strcmp((CHAR *)Str1, (CHAR *)Str2)
#define	CMP_MAC(a, b)	\
  ((*((ulong *)(((char *)(a)) + 2)) == *((ulong *)(((char *)(b)) + 2))) && \
  (*((ushort *)(a)) == *((ushort *)(b))))

//#define	CPY_MAC(Src, Dst, Len)	strncpy((CHAR *)(Dst), (CHAR *)(Src), (Len))
#define	CPY_MAC(s, d)	\
  *((ulong *)(d)) = *((ulong *)(s)), \
  *((ushort *)(((char *)(d)) + 4)) = *((ushort *)(((char *)(s)) + 4))

#define	NsmCompare(Str1, Str2, Len) 	strncmp((CHAR *)Str1, (CHAR *)Str2, Len)

#define	NsmIndicateLinkStatus(pAdapter)

#define	NSMGETNUMFRAGS(NumFrags, Ptr)	((NumFrags) = (Ptr)->NumFrags)

#define NSMMAPPKTS(pAdapter, pPkts, prio, NumDesc, PktIndex)


VOID 	NsmMarkRcbFree(HsmRCB *pRcb) ;
VOID 	NsmFreeContiguous(NsmContext *pNsm, AddrStruct *pAddr, UINT cached);
PVOID 	NsmMallocContiguous(NsmContext *pNsm, AddrStruct *pAddr, UINT align, UINT cached);
VOID	NsmTxCompleted(AdapterContext	*pAdapter, HsmTCB *pTCBHead, 
			HsmTCB *pTCBNext);
UCHAR 	NsmAllocatePacket(NsmContext *pNsmContext, HsmRCB *pRCB,AddrStruct *pAddr );
VOID 	NsmDeAllocatePacket(NsmContext *pNsmContext, AddrStruct *pAddr, 
			HsmRCB *pRCB);
VOID 	NsmRxPackets(AdapterContext *pAdapterContext, HsmRCB	*pRCBhead, 
				HsmRCB *pRCBnext, UINT pri);
PVOID 	NsmCreateLock();
